package com.sun.jersey.samples.helloworld;

import com.sun.jersey.api.core.ResourceConfig;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.ws.rs.WebApplicationException;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;
import org.osgi.service.event.EventConstants;
import org.osgi.service.event.EventHandler;

/**
 * @author Jakub.Podlesak@Sun.COM
 */
public class WebAppContextListener implements BundleActivator, ServletContextListener {

    static EventAdmin ea;
    public static ReloadServletContainer.Reloader reloader = new ReloadServletContainer.Reloader();
    public static ResourceConfig rc;
    BundleContext bc;
    ServiceReference eaRef;

    synchronized static EventAdmin getEa() {
        return ea;
    }

    synchronized static void setEa(EventAdmin ea) {
        WebAppContextListener.ea = ea;
    }

    @Override
    public void contextInitialized(final ServletContextEvent sce) {
        if (getEa() != null) {
            getEa().sendEvent(new Event("jersey/test/DEPLOYED", new HashMap<String, String>() {

                {
                    put("context-path", sce.getServletContext().getContextPath());
                }
            }));
        }
    }

    @Override
    public void contextDestroyed(final ServletContextEvent sce) {
        if (getEa() != null) {
            getEa().sendEvent(new Event("jersey/test/UNDEPLOYED", new HashMap<String, String>() {

                {
                    put("context-path", sce.getServletContext().getContextPath());
                }
            }));
            
        }
    }

    @Override
    public void start(BundleContext context) throws Exception {
        bc = context;
        eaRef = bc.getServiceReference(EventAdmin.class.getName());
        if (eaRef != null) {
            setEa((EventAdmin) bc.getService(eaRef));
            bc.registerService(EventHandler.class.getName(), new WebEventHandler("New Resource Handler"), getHandlerServiceProperties("jersey/new/Resource"));
        }
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        if (eaRef != null) {
            setEa(null);
            bc.ungetService(eaRef);
        }
    }

    public class WebEventHandler implements EventHandler {

        @Override
        public void handleEvent(Event event) {
            String nr = (String)event.getProperty("resource");
            String action = (String)event.getProperty("action");
            try {
                if ("add".equals(action)) {
                    rc.getClasses().add(Class.forName(nr));
                } else {
                    rc.getClasses().remove(Class.forName(nr));
                }
            } catch (ClassNotFoundException ex) {
                Logger.getLogger(WebAppContextListener.class.getName()).log(Level.SEVERE, null, ex);
                throw new WebApplicationException(ex);
            }
            reloader.reload();
        }

        public WebEventHandler(String handlerName) {
            this.handlerName = handlerName;
        }
        private final String handlerName;

        protected String getHandlerName() {
            return handlerName;
        }
    }

    private Dictionary getHandlerServiceProperties(String... topics) {
        Dictionary result = new Hashtable();
        result.put(EventConstants.EVENT_TOPIC, topics);
        return result;
    }
}
